@@ -223,7 +223,8 @@ class IndentedContextualConfig(dict):
             self._parser[cmd_class.context].append(cmd_class())
 
         except:
-            print("with: cmd_class=" + repr(cmd_class), file=sys.stderr)
+            print("add_command() with: cmd_class=" + repr(cmd_class),
+                  file=sys.stderr)
             raise
 
 
@@ -364,7 +365,7 @@ class IndentedContextualConfig(dict):
                             context["config"], **match.groupdict())
 
                     except:
-                        print("with: type(cmd)=" + repr(type(cmd)),
+                        print("parse() with: type(cmd)=" + repr(type(cmd)),
                               file=sys.stderr)
 
                         raise
