@@ -4,6 +4,15 @@
 
 
 
+"""Configuration differences converter module.
+
+This module contains abstract classes and functions to convert
+configurations (as a whole and individual elements) given the
+differences between two configurations.
+"""
+
+
+
 # --- imports ---
 
 
@@ -122,6 +131,19 @@ class DiffConvert:
         the repeated processing of the converter matches.
         """
 
+        # to avoid confusing error messages, just check all the
+        # definitions for this converter class are all tuples and not
+        # something like simple strings
+        if not (isinstance(self.context, tuple)):
+            raise TypeError("class %s: definition not tuple: context"
+                                % repr(type(self)))
+        if not (isinstance(self.cmd, tuple)):
+            raise TypeError("class %s: definition not tuple: cmd"
+                                % repr(type(self)))
+        if not (isinstance(self.ext, tuple)):
+            raise TypeError("class %s: definition not tuple: ext"
+                                % repr(type(self)))
+
         # calculate and store a few things, for efficiency
         self._path_full = self.context + self.cmd + self.ext
         self._context_len = len(self.context)
@@ -217,30 +239,61 @@ class DiffConvert:
         """This method takes a remove dictionary, as returned by
         deepdiff(), and a specific match into it (which must be from
         the same DiffConvert object as the method is called on) and
-        returns True iff the context for this converter is in it
-        entirely and exactly.
+        returns True iff the context for this converter is either:
+
+        1.  In the remove dictionary entirely and exactly (i.e. it is
+        empty at the end of the match path), or
+
+        2.  The dictionary does not contain the match path but the match
+        path runs out at a point in the dictionary where it is empty
+        (indicating everything below this is removed).
         """
 
         # if this converter has no containing context, we definitely
-        # can't be removing it, so will need to remove this item
+        # can't be removing it, so will need to remove the item
         if not self.context:
             return False
 
-        # get the part of the match fitting the context for this
-        # converter
+        # get the part of the match for this item which covers the
+        # context of this converter
         match_context = match[0 : self._context_len]
 
-        # see if the context is matched in its entirety in the remove
-        # dictionary - if it's not empty, or we get an error doing it,
-        # we're not removing the context
+        # loop whilst there is more match remaining and the remove
+        # dictionary still has items in it
+        while match_context and d:
+            # get the head of the match path and remove it from the path
+            # if we iterate after matching
+            match_head = match_context.pop(0)
+
+            # if the the head of the match path is not in the
+            # dictionary, we're not its context so return no match
+            if match_head not in d:
+                return False
+
+            # move down to the next level in the remove dictionary
+            d = d[match_head]
+
+
+        # if the remove dictionary is not empty, we've either reached
+        # the end of the match path and something remains, so we won't
+        # be removing the entire context, or the match path was not
+        # completely traversed but something remains
         #
-        # KeyError indicates item not found; TypeError if level is not
-        # a dictionary
-        try:
-            return deepget(d, *match_context, default_error=True) is None
-        except (KeyError, TypeError):
+        # either way, this converter will not be removing the context of
+        # this item so return no match
+        if d:
             return False
 
+        # the remove dictionary is empty, so we've either reached the
+        # end of the match path and are removing everything, or we ran
+        # out of remove dictionary, traversing the path, but everything
+        # at this point is going
+        #
+        # regardless, this converter will already be removing the
+        # context of this match so this item doesn't need to be
+        # explicitly removed, itself
+        return True
+
 
     def remove(self, old, *args):
         """The remove() method is called when the specified path is in
@@ -465,7 +518,8 @@ class DiffConfig:
 
 
     def __init__(self, init_explain=False, init_dump_config=False,
-                 init_dump_diff=False, init_debug_convert=0):
+                 init_dump_diff=False, init_debug_convert=0,
+                 init_subtree_dump_filter=[]):
 
         """The constructor initialises the exclude list to empty and
         adds and converters using _add_converters().  It also stores
@@ -492,6 +546,7 @@ class DiffConfig:
         self._dump_config = init_dump_config
         self._dump_diff = init_dump_diff
         self._debug_convert = init_debug_convert
+        self._subtree_dump_filter = init_subtree_dump_filter
 
         # initialise the dictionary of excludes, which will be passed
         # to deepremoveitems()
@@ -515,7 +570,7 @@ class DiffConfig:
 
 
     def _add_converter(self, cvt):
-        """Add an individual convert object, a child of the
+        """Add an individual converter object, a child of the
         DiffConverter class, to the list of converters to be used by the
         convert() method.
 
@@ -788,7 +843,9 @@ class DiffConfig:
 
         if self._dump_config:
             print(">> 'from' configuration (after rules, if specified):",
-                  yaml.dump(dict(from_cfg), default_flow_style=False),
+                  yaml.dump(deepget(dict(from_cfg),
+                                    *self._subtree_dump_filter),
+                            default_flow_style=False),
                   sep="\n")
 
 
@@ -814,7 +871,10 @@ class DiffConfig:
 
         if self._dump_config:
             print(">> 'to' configuration (after rules, if specified):",
-                yaml.dump(dict(to_cfg), default_flow_style=False), sep="\n")
+                  yaml.dump(deepget(dict(to_cfg),
+                                    *self._subtree_dump_filter),
+                            default_flow_style=False),
+                  sep="\n")
 
 
         # use deepdiff() to work out the differences between the two
@@ -831,13 +891,19 @@ class DiffConfig:
 
         if self._dump_diff:
             print("=> differences - remove:",
-                  yaml.dump(dict(remove_cfg), default_flow_style=False),
+                  yaml.dump(deepget(dict(remove_cfg),
+                                    *self._subtree_dump_filter),
+                            default_flow_style=False),
 
                   "=> differences - remove full:",
-                  yaml.dump(dict(remove_cfg_full), default_flow_style=False),
+                  yaml.dump(deepget(dict(remove_cfg_full),
+                                    *self._subtree_dump_filter),
+                            default_flow_style=False),
 
                   "=> differences - update (add/change):",
-                  yaml.dump(dict(update_cfg), default_flow_style=False),
+                  yaml.dump(deepget(dict(update_cfg),
+                                    *self._subtree_dump_filter),
+                            default_flow_style=False),
 
                   sep="\n")
 
@@ -895,7 +961,7 @@ class DiffConfig:
                     # in the 'to' configuration - if it does, we're
                     # doing a partial removal
 
-                    remove_is_trunc = False
+                    remove_is_truncate = False
 
                     try:
                         # we don't want the result here, just to find
@@ -911,7 +977,7 @@ class DiffConfig:
                         # something remains so this is partial
                         if self._debug_convert:
                             print("-> subconfiguration not empty - truncate")
-                        remove_is_trunc = True
+                        remove_is_truncate = True
 
 
                     # get the 'from' and 'remove' parts of the
@@ -942,14 +1008,17 @@ class DiffConfig:
                     # corresponding entry from the 'from' configuration
 
                     try:
-                        if remove_is_trunc:
+                        if remove_is_truncate:
                             diff = cvt.truncate(cvt_old, cvt_rem, *args)
                         else:
                             diff = cvt.remove(cvt_old, *args)
 
                     except:
-                        print("with: type(cvt)=" + repr(type(cvt)),
-                              file=sys.stderr)
+                        print("%s() with:" % ("truncate" if remove_is_truncate
+                                                  else "remove"),
+                              "  type(cvt)=" + repr(type(cvt)),
+                              "  *args=" + repr(args),
+                              sep="\n", file=sys.stderr)
 
                         raise
 
@@ -1056,8 +1125,11 @@ class DiffConfig:
                             diff = cvt.update(cvt_old, cvt_upd, cvt_new, *args)
 
                     except:
-                        print("with: type(cvt)=" + repr(type(cvt)),
-                              file=sys.stderr)
+                        print("%s() with:" % ("add" if update_is_add
+                                                  else "update"),
+                              "  type(cvt)=" + repr(type(cvt)),
+                              "  *args=" + repr(args),
+                              sep="\n", file=sys.stderr)
 
                         raise
 
