@@ -357,8 +357,7 @@ class Cmd_Int_ServPol(CmdContext_Int):
              r" (?P<policy>\S+)")
 
     def parse(self, cfg, type_, dir_, policy):
-        deepsetdefault(cfg, "service-policy", dir_, type_,
-                       last=policy)
+        deepsetdefault(cfg, "service-policy", dir_)[type_] = policy
 
 
 class Cmd_Int_Shutdown(CmdContext_Int):
@@ -651,7 +650,7 @@ class Cmd_RouteOSPF_AreaNSSA(CmdContext_RouterOSPF):
         a = set()
         if no_redist: a.add("no-redistribution")
         if no_summ: a.add("no-summary")
-        deepsetdefault(cfg, "area", area, "nssa", last=a)
+        deepsetdefault(cfg, "area", area)["nssa"] = a
 
 
 class Cmd_RouteOSPF_PasvInt(CmdContext_RouterOSPF):
@@ -663,8 +662,8 @@ class Cmd_RouteOSPF_PasvInt(CmdContext_RouterOSPF):
             if not no:
                 p["default"] = True
         else:
-            deepsetdefault(p, "interface", interface_canonicalize(int_name),
-                           last=not no)
+            deepsetdefault(
+                p, "interface")[interface_canonicalize(int_name)] = not no
 
 
 
